@@ -59,7 +59,7 @@ static GtkWidget *create_entry (GdauiEntryWrapper *mgwrap);
 static void       real_set_value (GdauiEntryWrapper *mgwrap, const GValue *value);
 static GValue    *real_get_value (GdauiEntryWrapper *mgwrap);
 static void       connect_signals(GdauiEntryWrapper *mgwrap, GCallback modify_cb, GCallback activate_cb);
-static gboolean   expand_in_layout (GdauiEntryWrapper *mgwrap);
+static gboolean   can_expand (GdauiEntryWrapper *mgwrap, gboolean horiz);
 static void       set_editable (GdauiEntryWrapper *mgwrap, gboolean editable);
 static void       grab_focus (GdauiEntryWrapper *mgwrap);
 
@@ -137,7 +137,7 @@ gdaui_entry_common_time_class_init (GdauiEntryCommonTimeClass * class)
 	GDAUI_ENTRY_WRAPPER_CLASS (class)->real_set_value = real_set_value;
 	GDAUI_ENTRY_WRAPPER_CLASS (class)->real_get_value = real_get_value;
 	GDAUI_ENTRY_WRAPPER_CLASS (class)->connect_signals = connect_signals;
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->expand_in_layout = expand_in_layout;
+	GDAUI_ENTRY_WRAPPER_CLASS (class)->can_expand = can_expand;
 	GDAUI_ENTRY_WRAPPER_CLASS (class)->set_editable = set_editable;
 	GDAUI_ENTRY_WRAPPER_CLASS (class)->grab_focus = grab_focus;
 
@@ -362,8 +362,10 @@ real_set_value (GdauiEntryWrapper *mgwrap, const GValue *value)
 	}
 	else if (type == GDA_TYPE_TIMESTAMP) {
 		if (value) {
-			if (gda_value_is_null ((GValue *) value))
+			if (gda_value_is_null ((GValue *) value)) {
+				gdaui_entry_set_text (GDAUI_ENTRY (mgtim->priv->entry_date), NULL);
 				gdaui_entry_set_text (GDAUI_ENTRY (mgtim->priv->entry_time), NULL);
+			}
 			else {
 				gchar *str, *ptr;
 				
@@ -410,12 +412,18 @@ real_get_value (GdauiEntryWrapper *mgwrap)
 
 	if (type == G_TYPE_DATE) {
 		str2 = gdaui_formatted_entry_get_text (GDAUI_FORMATTED_ENTRY (mgtim->priv->entry_date));
-		value = gda_data_handler_get_value_from_str (dh, str2, type); /* FIXME: not SQL but STR */
-		g_free (str2);
+		if (str2) {
+			value = gda_data_handler_get_value_from_str (dh, str2, type);
+			g_free (str2);
+		}
 	}
 	else if (type == GDA_TYPE_TIME) {
 		str2 = gdaui_formatted_entry_get_text (GDAUI_FORMATTED_ENTRY (mgtim->priv->entry_time));
-		value = gda_data_handler_get_value_from_str (dh, str2, type);
+		if (str2) {
+			value = gda_data_handler_get_value_from_str (dh, str2, type);
+			g_free (str2);
+		}
+
 		if (value && (G_VALUE_TYPE (value) != GDA_TYPE_NULL) &&
 		    mgtim->priv->last_value_set && 
 		    gda_value_isa (mgtim->priv->last_value_set, GDA_TYPE_TIME)) {
@@ -427,18 +435,19 @@ real_get_value (GdauiEntryWrapper *mgwrap)
 			gda_value_set_time (value, gdatime);
 			g_free (gdatime);
 		}
-		g_free (str2);
 	}
 	else if (type == GDA_TYPE_TIMESTAMP) {
 		gchar *tmpstr, *tmpstr2;
 
 		tmpstr = gdaui_formatted_entry_get_text (GDAUI_FORMATTED_ENTRY (mgtim->priv->entry_time));
 		tmpstr2 = gdaui_formatted_entry_get_text (GDAUI_FORMATTED_ENTRY (mgtim->priv->entry_date));
-		str2 = g_strdup_printf ("%s %s", tmpstr2, tmpstr);
+		if (tmpstr && tmpstr2) {
+			str2 = g_strdup_printf ("%s %s", tmpstr2, tmpstr);
+			value = gda_data_handler_get_value_from_str (dh, str2, type);
+			g_free (str2);
+		}
 		g_free (tmpstr);
 		g_free (tmpstr2);
-		value = gda_data_handler_get_value_from_str (dh, str2, type);
-		g_free (str2);
 		if (value && (G_VALUE_TYPE (value) != GDA_TYPE_NULL) &&
 		    mgtim->priv->last_value_set && 
 		    gda_value_isa (mgtim->priv->last_value_set, GDA_TYPE_TIMESTAMP)) {
@@ -510,7 +519,7 @@ connect_signals (GdauiEntryWrapper *mgwrap, GCallback modify_cb, GCallback activ
 }
 
 static gboolean
-expand_in_layout (GdauiEntryWrapper *mgwrap)
+can_expand (GdauiEntryWrapper *mgwrap, gboolean horiz)
 {
 	return FALSE;
 }
@@ -525,9 +534,9 @@ set_editable (GdauiEntryWrapper *mgwrap, gboolean editable)
 	g_return_if_fail (mgtim->priv);
 
 	if (mgtim->priv->entry_date)
-		gtk_entry_set_editable (GTK_ENTRY (mgtim->priv->entry_date), editable);
+		gtk_editable_set_editable (GTK_EDITABLE (mgtim->priv->entry_date), editable);
 	if (mgtim->priv->entry_time)
-		gtk_entry_set_editable (GTK_ENTRY (mgtim->priv->entry_time), editable);
+		gtk_editable_set_editable (GTK_EDITABLE (mgtim->priv->entry_time), editable);
 	if (mgtim->priv->date_button)
 		gtk_widget_set_sensitive (mgtim->priv->date_button, editable);
 }
