@@ -1,6 +1,6 @@
 /* gda-set.c
  *
- * Copyright (C) 2003 - 2008 Vivien Malerba
+ * Copyright (C) 2003 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -36,7 +36,7 @@
 #include "gda-server-provider.h"
 #include "gda-util.h"
 #include <libgda/gda-custom-marshal.h>
-#include <libgda/gda-error.h>
+#include <libgda/gda-types.h>
 
 extern xmlDtdPtr gda_paramlist_dtd;
 extern gchar *gda_lang_locale;
@@ -52,17 +52,19 @@ static void gda_set_finalize (GObject *object);
 static void set_remove_node (GdaSet *set, GdaSetNode *node);
 static void set_remove_source (GdaSet *set, GdaSetSource *source);
 
+
 static void changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
 static GError *validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *dataset);
 static void source_changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
 static void att_holder_changed_cb (GdaHolder *holder, const gchar *att_name, const GValue *att_value, GdaSet *dataset);
+static void holder_notify_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *dataset);
+
 
 static void compute_public_data (GdaSet *set);
 static gboolean gda_set_real_add_holder (GdaSet *set, GdaHolder *holder);
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
-extern GdaAttributesManager *gda_holder_attributes_manager;
 
 /* properties */
 enum
@@ -82,10 +84,12 @@ enum
 	HOLDER_ATTR_CHANGED,
 	VALIDATE_HOLDER_CHANGE,
 	VALIDATE_SET,
+	HOLDER_TYPE_SET,
+	SOURCE_MODEL_CHANGED,
 	LAST_SIGNAL
 };
 
-static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
+static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0 };
 
 
 /* private structure */
@@ -95,6 +99,8 @@ struct _GdaSetPrivate
 	gchar           *name;
 	gchar           *descr;
 	GHashTable      *holders_hash; /* key = GdaHoler ID, value = GdaHolder */
+	GArray          *holders_array;
+	gboolean         read_only;
 };
 
 static void 
@@ -242,8 +248,8 @@ gda_set_class_init (GdaSetClass *class)
 			      GDA_TYPE_HOLDER);
 
 	/**
-	 * GdaSet::validate-holder-change:
-	 * @set: the object which received the signal
+	 * GdaSet::validate-holder-change
+	 * @set: the #GdaSet
 	 * @holder: the #GdaHolder which is going to change
 	 * @new_value: the proposed new value for @holder
 	 * 
@@ -262,8 +268,8 @@ gda_set_class_init (GdaSetClass *class)
 			      _gda_marshal_ERROR__OBJECT_VALUE, GDA_TYPE_ERROR, 2,
 			      GDA_TYPE_HOLDER, G_TYPE_VALUE);
 	/**
-	 * GdaSet::validate-set:
-	 * @set: the object which received the signal
+	 * GdaSet::validate-set
+	 * @set: the #GdaSet
 	 * 
 	 * Gets emitted when gda_set_is_valid() is called, use
 	 * this signal to control which combination of values @set's holder can have (for example to implement some business rules)
@@ -278,6 +284,15 @@ gda_set_class_init (GdaSetClass *class)
 			      G_STRUCT_OFFSET (GdaSetClass, validate_set),
 			      validate_accumulator, NULL,
 			      _gda_marshal_ERROR__VOID, GDA_TYPE_ERROR, 0);
+	/**
+	 * GdaSet::holder-attr-changed
+	 * @set: the #GdaSet
+	 * @holder: the GdaHolder for which an attribute changed
+	 * @attr_name: attribute's name
+	 * @attr_value: attribute's value
+	 * 
+	 * Gets emitted when an attribute for any of the #GdaHolder objects managed by @set has changed
+	 */
 	gda_set_signals[HOLDER_ATTR_CHANGED] =
 		g_signal_new ("holder-attr-changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -286,6 +301,12 @@ gda_set_class_init (GdaSetClass *class)
 			      NULL, NULL,
 			      _gda_marshal_VOID__OBJECT_STRING_VALUE, G_TYPE_NONE, 3,
 			      GDA_TYPE_HOLDER, G_TYPE_STRING, G_TYPE_VALUE);
+	/**
+	 * GdaSet::public-data-changed
+	 * @set: the #GdaSet
+	 * 
+	 * Gets emitted when @set's public data (#GdaSetNode, #GdaSetGroup or #GdaSetSource values) have changed
+	 */
 	gda_set_signals[PUBLIC_DATA_CHANGED] =
 		g_signal_new ("public-data-changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -294,27 +315,67 @@ gda_set_class_init (GdaSetClass *class)
 			      NULL, NULL,
 			      _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
+	/**
+	 * GdaSet::holder-type-set
+	 * @set: the #GdaSet
+	 * @holder: the #GdaHolder for which the #GType has been set
+	 *
+	 * Gets emitted when @holder in @set has its type finally set, in case
+	 * it was #GDA_TYPE_NULL
+	 *
+	 * Since: 4.2
+	 */
+	gda_set_signals[HOLDER_TYPE_SET] =
+		g_signal_new ("holder-type-set",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GdaSetClass, holder_type_set),
+			      NULL, NULL,
+			      _gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      GDA_TYPE_HOLDER);
+
+	/**
+	 * GdaSet::source-model-changed
+	 * @set: the #GdaSet
+	 * @source: the #GdaSetSource for which the @data_model attribute has changed
+	 *
+	 * Gets emitted when the data model in @source has changed
+	 *
+	 * Since: 4.2
+	 */
+	gda_set_signals[SOURCE_MODEL_CHANGED] =
+		g_signal_new ("source-model-changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GdaSetClass, source_model_changed),
+			      NULL, NULL,
+			      _gda_marshal_VOID__POINTER, G_TYPE_NONE, 1,
+			      G_TYPE_POINTER);
+
+
 	class->holder_changed = NULL;
 	class->validate_holder_change = m_validate_holder_change;
 	class->validate_set = m_validate_set;
 	class->holder_attr_changed = NULL;
 	class->public_data_changed = NULL;
+	class->holder_type_set = NULL;
+	class->source_model_changed = NULL;
 
 	/* Properties */
 	object_class->set_property = gda_set_set_property;
 	object_class->get_property = gda_set_get_property;
 	g_object_class_install_property (object_class, PROP_ID,
-					 g_param_spec_string ("id", NULL, NULL, NULL, 
+					 g_param_spec_string ("id", NULL, "Id", NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_NAME,
-					 g_param_spec_string ("name", NULL, NULL, NULL, 
+					 g_param_spec_string ("name", NULL, "Name", NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_DESCR,
-					 g_param_spec_string ("description", NULL, NULL, NULL, 
+					 g_param_spec_string ("description", NULL, "Description", NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_HOLDERS,
 					 g_param_spec_pointer ("holders", "GSList of GdaHolders", 
-							       "Holders the set should contain", (
+							       "GdaHolder objects the set should contain", (
 								G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
   
 	object_class->dispose = gda_set_dispose;
@@ -330,12 +391,14 @@ gda_set_init (GdaSet *set)
 	set->sources_list = NULL;
 	set->groups_list = NULL;
 	set->priv->holders_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	set->priv->holders_array = NULL;
+	set->priv->read_only = FALSE;
 }
 
 
 /**
- * gda_set_new
- * @holders: a list of #GdaHolder objects
+ * gda_set_new:
+ * @holders: (element-type GdaHolder) (transfer:none): a list of #GdaHolder objects
  *
  * Creates a new #GdaSet object, and populates it with the list given as argument.
  * The list can then be freed as it is copied. All the value holders in @holders are referenced counted
@@ -348,16 +411,38 @@ gda_set_new (GSList *holders)
 {
 	GObject *obj;
 
-	obj = g_object_new (GDA_TYPE_SET, "holders", holders, NULL);
-	
-	return GDA_SET (obj);
+	obj = g_object_new (GDA_TYPE_SET, NULL);
+	for (; holders; holders = holders->next) 
+		gda_set_real_add_holder ((GdaSet*) obj, GDA_HOLDER (holders->data));
+	compute_public_data ((GdaSet*) obj);
+
+	return (GdaSet*) obj;
 }
 
+/*
+ * _gda_set_new_read_only
+ */
+GdaSet *
+_gda_set_new_read_only (GSList *holders)
+{
+	GObject *obj;
+
+	obj = g_object_new (GDA_TYPE_SET, NULL);
+	((GdaSet*) obj)->priv->read_only = TRUE;
+	for (; holders; holders = holders->next) 
+		gda_set_real_add_holder ((GdaSet*) obj, GDA_HOLDER (holders->data));
+	compute_public_data ((GdaSet*) obj);
+
+	return (GdaSet*) obj;
+}
+
+
+
 /**
- * gda_set_copy
+ * gda_set_copy:
  * @set: a #GdaSet object
  *
- * Creates a new #GdaSet object, opy of @set
+ * Creates a new #GdaSet object, copy of @set
  *
  * Returns: a new #GdaSet object
  */
@@ -380,7 +465,7 @@ gda_set_copy (GdaSet *set)
 }
 
 /**
- * gda_set_new_inline
+ * gda_set_new_inline:
  * @nb: the number of value holders which will be contained in the new #GdaSet
  * @...: a serie of a (const gchar*) id, (GType) type, and value
  *
@@ -388,7 +473,7 @@ gda_set_copy (GdaSet *set)
  * For each triplet (id, Glib type and value), 
  * the value must be of the correct type (gchar * if type is G_STRING, ...)
  *
- * Note that this function is a utility function and that anly a limited set of types are supported. Trying
+ * Note that this function is a utility function and that only a limited set of types are supported. Trying
  * to use an unsupported type will result in a warning, and the returned value holder holding a safe default
  * value.
  *
@@ -487,7 +572,7 @@ gda_set_new_inline (gint nb, ...)
 }
 
 /**
- * gda_set_set_holder_value
+ * gda_set_set_holder_value:
  * @set: a #GdaSet object
  * @error: a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
@@ -568,7 +653,7 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
 }
 
 /**
- * gda_set_get_holder_value
+ * gda_set_get_holder_value:
  * @set: a #GdaSet object
  * @holder_id: the ID of the holder to set the value
  *
@@ -615,7 +700,7 @@ xml_validity_error_func (void *ctx, const char *msg, ...)
 }
 
 /**
- * gda_set_new_from_spec_string
+ * gda_set_new_from_spec_string:
  * @xml_spec: a string
  * @error: a place to store the error, or %NULL
  *
@@ -707,7 +792,7 @@ gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
 
 
 /**
- * gda_set_new_from_spec_node
+ * gda_set_new_from_spec_node:
  * @xml_spec: a #xmlNodePtr for a &lt;holders&gt; tag
  * @error: a place to store the error, or %NULL
  *
@@ -758,9 +843,9 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 				}
 				else  {
 					sources = g_slist_prepend (sources, model);
-					str = (gchar*)xmlGetProp(cur, (xmlChar*)"name");
+					str = (gchar*)xmlGetProp(cur, (xmlChar*) "name");
 					if (str) 
-						g_object_set_data_full (G_OBJECT (model), "name", str, g_free);
+						g_object_set_data_full (G_OBJECT (model), "name", str, xmlFree);
 				}
 			}
 		}
@@ -867,9 +952,11 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 }
 
 /**
- * gda_set_remove_holder
- * @set:
- * @holder:
+ * gda_set_remove_holder:
+ * @set: a #GdaSet object
+ * @holder: the #GdaHolder to remove from @set
+ *
+ * Removes a #GdaHolder from the list of holders managed by @set
  */
 void
 gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
@@ -880,14 +967,19 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 	g_return_if_fail (set->priv);
 	g_return_if_fail (g_slist_find (set->holders, holder));
 
-	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-					      G_CALLBACK (changed_holder_cb), set);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 					      G_CALLBACK (validate_change_holder_cb), set);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-					      G_CALLBACK (source_changed_holder_cb), set);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-					      G_CALLBACK (att_holder_changed_cb), set);
+	if (! set->priv->read_only) {
+		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+						      G_CALLBACK (changed_holder_cb), set);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+						      G_CALLBACK (source_changed_holder_cb), set);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+						      G_CALLBACK (att_holder_changed_cb), set);
+	}
+	if (gda_holder_get_g_type (holder) == GDA_TYPE_NULL)
+		g_signal_handlers_disconnect_by_func (holder,
+						      G_CALLBACK (holder_notify_cb), set);
 
 	/* now destroy the GdaSetNode and the GdaSetSource if necessary */
 	node = gda_set_get_node (set, holder);
@@ -905,6 +997,10 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 
 	set->holders = g_slist_remove (set->holders, holder);
 	g_hash_table_remove (set->priv->holders_hash, gda_holder_get_id (holder));
+	if (set->priv->holders_array) {
+		g_array_free (set->priv->holders_array, TRUE);
+		set->priv->holders_array = NULL;
+	}
 	g_object_unref (G_OBJECT (holder));
 }
 
@@ -931,13 +1027,17 @@ validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *set)
 {
 	/* signal the holder validate-change */
 	GError *error = NULL;
+	if (set->priv->read_only)
+		g_set_error (&error, GDA_SET_ERROR, GDA_SET_READ_ONLY_ERROR, _("Data set does not allow modifications"));
+	else {
 #ifdef GDA_DEBUG_signal
-	g_print (">> 'VALIDATE_HOLDER_CHANGE' from %s\n", __FUNCTION__);
+		g_print (">> 'VALIDATE_HOLDER_CHANGE' from %s\n", __FUNCTION__);
 #endif
-	g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_HOLDER_CHANGE], 0, holder, value, &error);
+		g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_HOLDER_CHANGE], 0, holder, value, &error);
 #ifdef GDA_DEBUG_signal
-	g_print ("<< 'VALIDATE_HOLDER_CHANGED' from %s\n", __FUNCTION__);
+		g_print ("<< 'VALIDATE_HOLDER_CHANGED' from %s\n", __FUNCTION__);
 #endif
+	}
 	return error;
 }
 
@@ -974,15 +1074,17 @@ gda_set_dispose (GObject *object)
 	/* free the holders list */
 	if (set->holders) {
 		for (list = set->holders; list; list = list->next) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-							      G_CALLBACK (changed_holder_cb), set);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
 							      G_CALLBACK (validate_change_holder_cb), set);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-							      G_CALLBACK (source_changed_holder_cb), set);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-							      G_CALLBACK (att_holder_changed_cb), set);
-			g_object_unref (G_OBJECT (list->data));
+			if (! set->priv->read_only) {
+				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+								      G_CALLBACK (changed_holder_cb), set);
+				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+								      G_CALLBACK (source_changed_holder_cb), set);
+				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+								      G_CALLBACK (att_holder_changed_cb), set);
+			}
+			g_object_unref (list->data);
 		}
 		g_slist_free (set->holders);
 	}
@@ -990,6 +1092,10 @@ gda_set_dispose (GObject *object)
 		g_hash_table_destroy (set->priv->holders_hash);
 		set->priv->holders_hash = NULL;
 	}
+	if (set->priv->holders_array) {
+		g_array_free (set->priv->holders_array, TRUE);
+		set->priv->holders_array = NULL;
+	}
 
 	/* free the nodes if there are some */
 	while (set->nodes_list)
@@ -1026,9 +1132,6 @@ gda_set_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
-static void compute_shown_columns_index (GdaSetSource *source);
-static void compute_ref_columns_index (GdaSetSource *source);
-
 /*
  * Resets and computes set->nodes, and if some nodes already exist, they are previously discarded
  */
@@ -1039,8 +1142,7 @@ compute_public_data (GdaSet *set)
 	GdaSetNode *node;
 	GdaSetSource *source;
 	GdaSetGroup *group;
-	GHashTable *groups = g_hash_table_new (NULL, NULL); /* key = source model, 
-							       value = GdaSetGroup */
+	GHashTable *groups = NULL;
 
 	/*
 	 * Get rid of all the previous structures
@@ -1065,14 +1167,14 @@ compute_public_data (GdaSet *set)
 		if (node->source_model)
 			g_object_ref (node->source_model);
 		
-		set->nodes_list = g_slist_append (set->nodes_list, node);
+		set->nodes_list = g_slist_prepend (set->nodes_list, node);
 	}
+	set->nodes_list = g_slist_reverse (set->nodes_list);
 
 	/*
 	 * Creation of the GdaSetSource and GdaSetGroup structures 
 	 */
-	list = set->nodes_list;
-	while (list) {
+	for (list = set->nodes_list; list;list = list->next) {
 		node = GDA_SET_NODE (list->data);
 		
 		/* source */
@@ -1080,18 +1182,18 @@ compute_public_data (GdaSet *set)
 		if (node->source_model) {
 			source = gda_set_get_source_for_model (set, node->source_model);
 			if (source) 
-				source->nodes = g_slist_prepend (source->nodes, node);
+				source->nodes = g_slist_append (source->nodes, node);
 			else {
 				source = g_new0 (GdaSetSource, 1);
 				source->data_model = node->source_model;
-				source->nodes = g_slist_prepend (NULL, node);
+				source->nodes = g_slist_append (NULL, node);
 				set->sources_list = g_slist_prepend (set->sources_list, source);
 			}
 		}
 
 		/* group */
 		group = NULL;
-		if (node->source_model)
+		if (node->source_model && groups)
 			group = g_hash_table_lookup (groups, node->source_model);
 		if (group) 
 			group->nodes = g_slist_append (group->nodes, node);
@@ -1099,25 +1201,18 @@ compute_public_data (GdaSet *set)
 			group = g_new0 (GdaSetGroup, 1);
 			group->nodes = g_slist_append (NULL, node);
 			group->nodes_source = source;
-			set->groups_list = g_slist_append (set->groups_list, group);
-			if (node->source_model)
+			set->groups_list = g_slist_prepend (set->groups_list, group);
+			if (node->source_model) {
+				if (!groups)
+					groups = g_hash_table_new (NULL, NULL); /* key = source model, 
+										   value = GdaSetGroup */
 				g_hash_table_insert (groups, node->source_model, group);
+			}
 		}		
-
-		list = g_slist_next (list);
 	}
-	
-	/*
-	 * Filling information in all the GdaSetSource structures
-	 */
-	list = set->sources_list;
-	while (list) {
-		compute_shown_columns_index (GDA_SET_SOURCE (list->data));
-		compute_ref_columns_index (GDA_SET_SOURCE (list->data));
-		list = g_slist_next (list);
-	}
-
-	g_hash_table_destroy (groups);
+	set->groups_list = g_slist_reverse (set->groups_list);
+	if (groups)
+		g_hash_table_destroy (groups);
 
 #ifdef GDA_DEBUG_signal
         g_print (">> 'PUBLIC_DATA_CHANGED' from %p\n", set);
@@ -1128,105 +1223,8 @@ compute_public_data (GdaSet *set)
 #endif
 }
 
-static void
-compute_shown_columns_index (GdaSetSource *source)
-{
-        gint ncols, nholders;
-        gint *mask = NULL, masksize = 0;
-
-        nholders = g_slist_length (source->nodes);
-        g_return_if_fail (nholders > 0);
-        ncols = gda_data_model_get_n_columns (GDA_DATA_MODEL (source->data_model));
-        g_return_if_fail (ncols > 0);
-
-        if (ncols > nholders) {
-                /* we only want columns which are not holders */
-                gint i, current = 0;
-
-                masksize = ncols - nholders;
-                mask = g_new0 (gint, masksize);
-                for (i = 0; i < ncols ; i++) {
-                        GSList *list = source->nodes;
-                        gboolean found = FALSE;
-                        while (list && !found) {
-                                if (GDA_SET_NODE (list->data)->source_column == i)
-                                        found = TRUE;
-                                else
-                                        list = g_slist_next (list);
-                        }
-                        if (!found) {
-                                mask[current] = i;
-                                current ++;
-                        }
-                }
-                masksize = current;
-        }
-        else {
-                /* we want all the columns */
-                gint i;
-
-                masksize = ncols;
-                mask = g_new0 (gint, masksize);
-                for (i=0; i<ncols; i++) {
-                        mask[i] = i;
-                }
-        }
-
-        source->shown_n_cols = masksize;
-        source->shown_cols_index = mask;
-}
-
-static void
-compute_ref_columns_index (GdaSetSource *source)
-{
-        gint ncols, nholders;
-        gint *mask = NULL, masksize = 0;
-
-        nholders = g_slist_length (source->nodes);
-        g_return_if_fail (nholders > 0);
-        ncols = gda_data_model_get_n_columns (GDA_DATA_MODEL (source->data_model));
-        g_return_if_fail (ncols > 0);
-
-        if (ncols > nholders) {
-                /* we only want columns which are holders */
-                gint i, current = 0;
-
-                masksize = ncols - nholders;
-                mask = g_new0 (gint, masksize);
-                for (i=0; i<ncols ; i++) {
-                        GSList *list = source->nodes;
-                        gboolean found = FALSE;
-                        while (list && !found) {
-                                if (GDA_SET_NODE (list->data)->source_column == i)
-                                        found = TRUE;
-                                else
-                                        list = g_slist_next (list);
-                        }
-                        if (found) {
-                                mask[current] = i;
-                                current ++;
-                        }
-                }
-                masksize = current;
-        }
-        else {
-                /* we want all the columns */
-                gint i;
-
-                masksize = ncols;
-                mask = g_new0 (gint, masksize);
-                for (i=0; i<ncols; i++) {
-                        mask[i] = i;
-                }
-        }
-
-        source->ref_n_cols = masksize;
-        source->ref_cols_index = mask;
-}
-
-
 /**
- * gda_set_add_holder
+ * gda_set_add_holder:
  * @set: a #GdaSet object
  * @holder: a #GdaHolder object
  *
@@ -1252,15 +1250,23 @@ gda_set_add_holder (GdaSet *set, GdaHolder *holder)
 	return added;
 }
 
+static void
+holder_notify_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *dataset)
+{
+	GType gtype;
+	gtype = gda_holder_get_g_type (holder);
+	g_assert (gtype != GDA_TYPE_NULL);
+	g_signal_handlers_disconnect_by_func (holder,
+					      G_CALLBACK (holder_notify_cb), dataset);
+	g_signal_emit (dataset, gda_set_signals[HOLDER_TYPE_SET], 0, holder);
+}
+
 static gboolean
 gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 {
 	GdaHolder *similar;
 	const gchar *hid;
 
-	if (g_slist_find (set->holders, holder))
-		return FALSE;
-
 	/* 
 	 * try to find a similar holder in the set->holders:
 	 * a holder B is similar to a holder A if it has the same ID
@@ -1270,23 +1276,35 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 		g_warning (_("GdaHolder needs to have an ID"));
 		return FALSE;
 	}
-	g_return_val_if_fail (hid, FALSE);
+
 	similar = (GdaHolder*) g_hash_table_lookup (set->priv->holders_hash, hid);
 	if (!similar) {
 		/* really add @holder to the set */
 		set->holders = g_slist_append (set->holders, holder);
 		g_hash_table_insert (set->priv->holders_hash, (gchar*) hid, holder);
+		if (set->priv->holders_array) {
+			g_array_free (set->priv->holders_array, TRUE);
+			set->priv->holders_array = NULL;
+		}
 		g_object_ref (holder);
-		g_signal_connect (G_OBJECT (holder), "changed",
-				  G_CALLBACK (changed_holder_cb), set);
 		g_signal_connect (G_OBJECT (holder), "validate-change",
 				  G_CALLBACK (validate_change_holder_cb), set);
-		g_signal_connect (G_OBJECT (holder), "source-changed",
-				  G_CALLBACK (source_changed_holder_cb), set);
-		g_signal_connect (G_OBJECT (holder), "attribute-changed",
-				  G_CALLBACK (att_holder_changed_cb), set);
+		if (! set->priv->read_only) {
+			g_signal_connect (G_OBJECT (holder), "changed",
+					  G_CALLBACK (changed_holder_cb), set);
+			g_signal_connect (G_OBJECT (holder), "source-changed",
+					  G_CALLBACK (source_changed_holder_cb), set);
+			g_signal_connect (G_OBJECT (holder), "attribute-changed",
+					  G_CALLBACK (att_holder_changed_cb), set);
+		}
+		if (gda_holder_get_g_type (holder) == GDA_TYPE_NULL)
+			g_signal_connect (G_OBJECT (holder), "notify::g-type",
+					  G_CALLBACK (holder_notify_cb), set);
+
 		return TRUE;
 	}
+	else if (similar == holder)
+		return FALSE;
 	else {
 #ifdef GDA_DEBUG_NO
 		g_print ("In Set %p, Holder %p and %p are similar, keeping %p only\n", set, similar, holder, similar);
@@ -1297,7 +1315,7 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 
 
 /**
- * gda_set_merge_with_set
+ * gda_set_merge_with_set:
  * @set: a #GdaSet object
  * @set_to_merge: a #GdaSet object
  *
@@ -1336,15 +1354,13 @@ set_remove_source (GdaSet *set, GdaSetSource *source)
 
 	if (source->nodes)
 		g_slist_free (source->nodes);
-	g_free (source->shown_cols_index);
-	g_free (source->ref_cols_index);
 
 	set->sources_list = g_slist_remove (set->sources_list, source);
 	g_free (source);
 }
 
 /**
- * gda_set_is_valid
+ * gda_set_is_valid:
  * @set: a #GdaSet object
  * @error: a place to store validation errors, or %NULL
  *
@@ -1372,6 +1388,12 @@ gda_set_is_valid (GdaSet *set, GError **error)
 		}
 	}
 
+	return _gda_set_validate (set, error);
+}
+
+gboolean
+_gda_set_validate (GdaSet *set, GError **error)
+{
 	/* signal the holder validate-set */
 	GError *lerror = NULL;
 #ifdef GDA_DEBUG_signal
@@ -1388,33 +1410,61 @@ gda_set_is_valid (GdaSet *set, GError **error)
 	return TRUE;
 }
 
+
 /**
- * gda_set_get_holder
+ * gda_set_get_holder:
  * @set: a #GdaSet object
  * @holder_id: the ID of the requested value holder
  *
  * Finds a #GdaHolder using its ID
  *
- * Returns: a #GdaHolder or %NULL
+ * Returns: (transfer none): the requested #GdaHolder or %NULL
  */
 GdaHolder *
 gda_set_get_holder (GdaSet *set, const gchar *holder_id)
 {
 	g_return_val_if_fail (GDA_IS_SET (set), NULL);
-	g_return_val_if_fail (set->priv, NULL);
+	g_return_val_if_fail (holder_id, NULL);
 
 	return (GdaHolder *) g_hash_table_lookup (set->priv->holders_hash, holder_id);
 }
 
+/**
+ * gda_set_get_nth_holder:
+ * @set: a #GdaSet object
+ * @pos: the position of the requested #GdaHolder, starting at %0
+ *
+ * Finds a #GdaHolder using its position
+ *
+ * Returns: (transfer none): the requested #GdaHolder or %NULL
+ *
+ * Since: 4.2
+ */
+GdaHolder *
+gda_set_get_nth_holder (GdaSet *set, gint pos)
+{
+	g_return_val_if_fail (GDA_IS_SET (set), NULL);
+	if (! set->priv->holders_array) {
+		GSList *list;
+		set->priv->holders_array = g_array_sized_new (FALSE, FALSE, sizeof (GdaHolder*),
+							      g_slist_length (set->holders));
+		for (list = set->holders; list; list = list->next)
+			g_array_append_val (set->priv->holders_array, list->data);
+	}
+	if ((pos < 0) || (pos > set->priv->holders_array->len))
+		return NULL;
+	else
+		return g_array_index (set->priv->holders_array, GdaHolder*, pos);
+}
 
 /**
- * gda_set_get_node
+ * gda_set_get_node:
  * @set: a #GdaSet object
  * @holder: a #GdaHolder object
  *
  * Finds a #GdaSetNode holding information for @holder, don't modify the returned structure
  *
- * Returns: a #GdaSetNode or %NULL
+ * Returns: (transfer none): the requested #GdaSetNode or %NULL
  */
 GdaSetNode *
 gda_set_get_node (GdaSet *set, GdaHolder *holder)
@@ -1436,14 +1486,14 @@ gda_set_get_node (GdaSet *set, GdaHolder *holder)
 }
 
 /**
- * gda_set_get_source
+ * gda_set_get_source:
  * @set: a #GdaSet object
  * @holder: a #GdaHolder object
  *
  * Finds a #GdaSetSource which contains the #GdaDataModel restricting the possible values of
  * @holder, don't modify the returned structure.
  *
- * Returns: a #GdaSetSource or %NULL
+ * Returns: (transfer none): the requested #GdaSetSource or %NULL
  */
 GdaSetSource *
 gda_set_get_source (GdaSet *set, GdaHolder *holder)
@@ -1458,14 +1508,14 @@ gda_set_get_source (GdaSet *set, GdaHolder *holder)
 }
 
 /**
- * gda_set_get_group
+ * gda_set_get_group:
  * @set: a #GdaSet object
  * @holder: a #GdaHolder object
  *
  * Finds a #GdaSetGroup which lists a  #GdaSetNode containing @holder,
  * don't modify the returned structure.
  *
- * Returns: a #GdaSetGroup or %NULL
+ * Returns: (transfer none): the requested #GdaSetGroup or %NULL
  */
 GdaSetGroup *
 gda_set_get_group (GdaSet *set, GdaHolder *holder)
@@ -1493,14 +1543,14 @@ gda_set_get_group (GdaSet *set, GdaHolder *holder)
 
 
 /**
- * gda_set_get_source_for_model
+ * gda_set_get_source_for_model:
  * @set: a #GdaSet object
  * @model: a #GdaDataModel object
  *
  * Finds the #GdaSetSource structure used in @set for which @model is a
- * the data model, don't modify the returned structure
+ * the data model (the returned structure should not be modified).
  *
- * Returns: a #GdaSetSource pointer or %NULL.
+ * Returns: (transfer none): the requested #GdaSetSource pointer or %NULL.
  */
 GdaSetSource *
 gda_set_get_source_for_model (GdaSet *set, GdaDataModel *model)
@@ -1522,3 +1572,73 @@ gda_set_get_source_for_model (GdaSet *set, GdaDataModel *model)
 
 	return retval;
 }
+
+/**
+ * gda_set_replace_source_model
+ * @set: a #GdaSet object
+ * @source: a pointer to a #GdaSetSource in @set
+ * @model: a #GdaDataModel
+ *
+ * Replaces @source->data_model with @model, which must have the same
+ * characteristics as @source->data_model (same column types)
+ *
+ * Also for each #GdaHolder for which @source->data_model is a source model,
+ * this method calls gda_holder_set_source_model() with @model to replace
+ * the source by the new model
+ *
+ * Since: 4.2
+ */
+void
+gda_set_replace_source_model (GdaSet *set, GdaSetSource *source, GdaDataModel *model)
+{
+	g_return_if_fail (GDA_IS_SET (set));
+	g_return_if_fail (source);
+	g_return_if_fail (g_slist_find (set->sources_list, source));
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+	
+	/* compare models */
+	gint ncols, i;
+	ncols = gda_data_model_get_n_columns (source->data_model);
+	if (ncols != gda_data_model_get_n_columns (model)) {
+		g_warning (_("Replacing data model must have the same characteristics as the "
+			     "data model it replaces"));
+		return;
+	}
+	for (i = 0; i < ncols; i++) {
+		GdaColumn *c1, *c2;
+		GType t1, t2;
+		c1 = gda_data_model_describe_column (source->data_model, i);
+		c2 = gda_data_model_describe_column (model, i);
+		t1 = gda_column_get_g_type (c1);
+		t2 = gda_column_get_g_type (c2);
+
+		if ((t1 != GDA_TYPE_NULL) && (t2 != GDA_TYPE_NULL) && (t1 != t2)) {
+			g_warning (_("Replacing data model must have the same characteristics as the "
+				     "data model it replaces"));
+			return;
+		}
+	}
+
+	/* actually swap the models */
+	GSList *list;
+	source->data_model = model;
+	for (list = source->nodes; list; list = list->next) {
+		GdaSetNode *node = (GdaSetNode*) list->data;
+		g_object_unref (node->source_model);
+		node->source_model = g_object_ref (model);
+		g_signal_handlers_block_by_func (G_OBJECT (node->holder),
+						 G_CALLBACK (source_changed_holder_cb), set);
+		gda_holder_set_source_model (GDA_HOLDER (node->holder), model, node->source_column,
+					     NULL);
+		g_signal_handlers_unblock_by_func (G_OBJECT (node->holder),
+						   G_CALLBACK (source_changed_holder_cb), set);
+
+	}
+#ifdef GDA_DEBUG_signal
+	g_print (">> 'SOURCE_MODEL_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (set), gda_set_signals[SOURCE_MODEL_CHANGED], 0, source);
+#ifdef GDA_DEBUG_signal
+	g_print ("<< 'SOURCE_MODEL_CHANGED' from %s\n", __FUNCTION__);
+#endif
+}
